@@ -27,9 +27,9 @@ OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 """
 
-import pysodium
 import unittest
 import binascii
+import pysodium
 
 
 class TestPySodium(unittest.TestCase):
@@ -59,6 +59,9 @@ class TestPySodium(unittest.TestCase):
         self.assertEqual(r, r1)
         self.assertEqual(r6, r61)
 
+        self.assertNotEqual(pysodium.crypto_generichash( 'salt0'), pysodium.crypto_generichash( 'salt1'))
+        self.assertNotEqual(pysodium.crypto_generichash(b'salt0'), pysodium.crypto_generichash(b'salt1'))
+
     def test_crypto_box_pk_from_sk(self):
         pk1, sk = pysodium.crypto_box_keypair()
         pk2 = pysodium.crypto_scalarmult_curve25519_base(sk)
@@ -88,6 +91,16 @@ class TestPySodium(unittest.TestCase):
         plaintext = pysodium.crypto_box_open_afternm(c, n, k)
         self.assertEqual(m, plaintext)
 
+    def test_crypto_secretbox_open_detached(self):
+        m = b"howdy"
+        n = pysodium.randombytes(pysodium.crypto_secretbox_NONCEBYTES)
+        k = pysodium.randombytes(pysodium.crypto_secretbox_KEYBYTES)
+        c, mac = pysodium.crypto_secretbox_detached(m, n, k)
+        mplain = pysodium.crypto_secretbox_open_detached(c, mac, n, k)
+        self.assertEqual(m, mplain)
+        changed = b"\0"*len(c)
+        self.assertRaises(ValueError, pysodium.crypto_secretbox_open_detached, changed, mac, n, k)
+
     def test_crypto_box_open_detached(self):
         pk, sk = pysodium.crypto_box_keypair()
         n = pysodium.randombytes(pysodium.crypto_box_NONCEBYTES)
@@ -260,7 +273,16 @@ class TestPySodium(unittest.TestCase):
         state2 = pysodium.crypto_secretstream_xchacha20poly1305_init_pull(header, key)
         self.assertRaises(ValueError, pysodium.crypto_secretstream_xchacha20poly1305_pull, state2, ciphertext2, None)
 
-    def test_test_crypto_scalarmult_curve25519_base(self):
+    def test_crypto_scalarmult_base(self):
+        # In the C code, crypto_scalarmult_base just delegates to
+        # crypto_scalarmult_curve25519_base. If libsodium changes the preferred
+        # algorithm, this answer will change.
+        k = binascii.unhexlify(b"e38e290880cee71a0cbb7b09328fd034c1fe4bd8838b19ab303a64a8c6b01456")
+        expected = binascii.unhexlify(b"4aa82c2514ed88eb46085369a45ddd0db997e53bfee877c4556ab49a1581e545")
+        actual = pysodium.crypto_scalarmult_base(k)
+        self.assertEqual(expected, actual)
+
+    def test_crypto_scalarmult_curve25519_base(self):
         s = pysodium.crypto_scalarmult_curve25519_base(pysodium.randombytes(pysodium.crypto_scalarmult_BYTES))
         r = pysodium.crypto_scalarmult_curve25519_base(pysodium.randombytes(pysodium.crypto_scalarmult_BYTES))
         pysodium.crypto_scalarmult_curve25519(s, r)
@@ -318,6 +340,21 @@ class TestPySodium(unittest.TestCase):
             output = pysodium.crypto_aead_chacha20poly1305_ietf_decrypt(output, ad, nonce, key)
             self.assertEqual(output, input_)
 
+    def test_aead_chacha20poly1305_ietf_detached(self):
+        if not pysodium.sodium_version_check(1, 0, 9): return
+        key = binascii.unhexlify(b"4290bcb154173531f314af57f3be3b5006da371ece272afa1b5dbdd1100a1007")
+        input_ = binascii.unhexlify(b"86d09974840bded2a5ca")
+        nonce = binascii.unhexlify(b"cd7cf67be39c794acd7cf67b")
+        for ad, ct in [
+            (binascii.unhexlify(b"87e229d4500845a079c0"), b"09394ed41cf16d3c0820c5e0caf8a7bf"),
+            (None, b"07bf99e3c0d8aaac48c04e1f93b12a63"),
+        ]:
+            output, mac = pysodium.crypto_aead_chacha20poly1305_ietf_encrypt_detached(input_, ad, nonce, key)
+            self.assertEqual(binascii.unhexlify(b"eef4c561bdda5ef7e044"), output)
+            self.assertEqual(binascii.unhexlify(ct), mac)
+            output = pysodium.crypto_aead_chacha20poly1305_ietf_decrypt_detached(output, mac, ad, nonce, key)
+            self.assertEqual(output, input_)
+
     def test_aead_xchacha20poly1305_ietf(self):
         if not pysodium.sodium_version_check(1, 0, 12): return
         key = binascii.unhexlify(b"4290bcb154173531f314af57f3be3b5006da371ece272afa1b5dbdd1100a1007")
@@ -336,6 +373,14 @@ class TestPySodium(unittest.TestCase):
         self.assertEqual(binascii.unhexlify(b"f798a189f195e66982105ffb640bb7757f579da31602fc93ec01ac56f85ac3c134a4547b733b46413042c9440049176905d3be59ea1c53f15916155c2be8241a38008b9a26bc35941e2444177c8ade6689de95264986d95889fb60e84629c9bd9a5acb1cc118be563eb9b3a4a472f82e09a7e778492b562ef7130e88dfe031c79db9d4f7c7a899151b9a475032b63fc385245fe054e3dd5a97a5f576fe064025d3ce042c566ab2c507b138db853e3d6959660996546cc9c4a6eafdc777c040d70eaf46f76dad3979e5c5360c3317166a1c894c94a371876a94df7628fe4eaaf2ccb27d5aaae0ad7ad0f9d4b6ad3b54098746d4524d38407a6deb3ab78fab78c9"),
                          output)
 
+    def test_crypto_stream_chacha20_xor_ic(self):
+        key = binascii.unhexlify(b"000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f")
+        nonce = binascii.unhexlify(b"0001020304050607")
+        input_ = b'\x00' * 128
+        ic = 2
+        output = pysodium.crypto_stream_chacha20_xor_ic(input_, nonce, ic, key)
+        self.assertEqual(binascii.unhexlify(b"9db9d4f7c7a899151b9a475032b63fc385245fe054e3dd5a97a5f576fe064025d3ce042c566ab2c507b138db853e3d6959660996546cc9c4a6eafdc777c040d70eaf46f76dad3979e5c5360c3317166a1c894c94a371876a94df7628fe4eaaf2ccb27d5aaae0ad7ad0f9d4b6ad3b54098746d4524d38407a6deb3ab78fab78c9"), output)
+
     def test_crypto_blake2b(self):
         message   = binascii.unhexlify(b'54686520717569636b2062726f776e20666f78206a756d7073206f76657220746865206c617a7920646f67')
         key       = binascii.unhexlify(b'000102030405060708090a0b0c0d0e0f')
@@ -456,18 +501,15 @@ class TestPySodium(unittest.TestCase):
         self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha512(b"\x80")),
             "dfe8ef54110b3324d3b889035c95cfb80c92704614bf76f17546ad4f4b08218a630e16da7df34766a975b3bb85b01df9e99a4ec0a1d0ec3de6bed7b7a40b2f10")
 
+    def test_crypto_hash_sha512_steps(self):
+        s = pysodium.crypto_hash_sha512_init()
+        pysodium.crypto_hash_sha512_update(s, b"Correct Horse ")
+        pysodium.crypto_hash_sha512_update(s, b"Battery Staple")
+        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha512_final(s)),
+            "0675070bda47bef936f0b65ae721d90f82ca137841df4d7cae27776501ae4b446ab926d64dc1d282c8758ac0eb02cc4aa11b2452d4f8ffeb795023b797fe2b80")
+
     def byteHashToString(self, input):
-        import sys
-        result = ""
-        for i in range(0, len(input)):
-            if sys.version_info.major == 3:
-                tmp = str(hex(ord(chr(input[i]))))[2:]
-            else:
-                tmp = str(hex(ord(input[i])))[2:]
-            if len(tmp) is 1:
-                tmp = "0" + tmp
-            result += tmp
-        return result
+        return binascii.hexlify(input).decode('utf8')
 
     def test_crypto_auth(self):
         sk = pysodium.randombytes(pysodium.crypto_auth_KEYBYTES)
@@ -485,5 +527,107 @@ class TestPySodium(unittest.TestCase):
         self.assertEqual(crx, stx)
         self.assertEqual(ctx, srx)
 
+    def test_sodium_inc(self):
+        r = b'A' * 32
+        pysodium.sodium_increment(r)
+        self.assertEqual(r, b'BAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA')
+
+    def test_crypto_core_ristretto255_scalar_random(self):
+        if not pysodium.sodium_version_check(1, 0, 18): return
+        a = pysodium.crypto_core_ristretto255_scalar_random()
+        b = pysodium.crypto_core_ristretto255_scalar_random()
+        # stupid check that random returns different values...
+        self.assertNotEqual(a,b)
+
+    def test_crypto_core_ristretto255_random(self):
+        if not pysodium.sodium_version_check(1, 0, 18): return
+        a = pysodium.crypto_core_ristretto255_random()
+        b = pysodium.crypto_core_ristretto255_random()
+        # same stupid check that random returns different values...
+        self.assertNotEqual(a,b)
+
+    def test_crypto_core_ristretto255_is_valid_point(self):
+        if not pysodium.sodium_version_check(1, 0, 18): return
+        invalid = binascii.unhexlify(b"ecffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff7f")
+        self.assertEqual(False, pysodium.crypto_core_ristretto255_is_valid_point(invalid))
+        invalid = binascii.unhexlify(b"ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff7f")
+        self.assertEqual(False, pysodium.crypto_core_ristretto255_is_valid_point(invalid))
+
+    def test_crypto_core_ristretto255_from_hash(self):
+        if not pysodium.sodium_version_check(1, 0, 18): return
+        h = pysodium.crypto_generichash(b'howdy', outlen=pysodium.crypto_core_ristretto255_HASHBYTES)
+        p = pysodium.crypto_core_ristretto255_from_hash(h)
+        pysodium.crypto_core_ristretto255_is_valid_point(p)
+
+    def test_crypto_scalarmult_ristretto255_base(self):
+        if not pysodium.sodium_version_check(1, 0, 18): return
+        p = pysodium.crypto_scalarmult_ristretto255_base(pysodium.crypto_core_ristretto255_scalar_random())
+        pysodium.crypto_core_ristretto255_is_valid_point(p)
+
+    def test_crypto_scalarmult_ristretto255(self):
+        if not pysodium.sodium_version_check(1, 0, 18): return
+        n = pysodium.crypto_scalarmult_ristretto255_base(pysodium.crypto_core_ristretto255_scalar_random())
+        p = pysodium.crypto_scalarmult_ristretto255_base(pysodium.crypto_core_ristretto255_scalar_random())
+        r = pysodium.crypto_scalarmult_ristretto255(n, p)
+        pysodium.crypto_core_ristretto255_is_valid_point(r)
+
+    def test_crypto_core_ristretto255_scalar_invert(self):
+        if not pysodium.sodium_version_check(1, 0, 18): return
+        s = pysodium.crypto_core_ristretto255_scalar_random()
+        r = pysodium.crypto_core_ristretto255_scalar_invert(s)
+        p = pysodium.crypto_scalarmult_ristretto255_base(pysodium.crypto_core_ristretto255_scalar_random())
+        q = pysodium.crypto_scalarmult_ristretto255(s, p)
+        p_ = pysodium.crypto_scalarmult_ristretto255(r, q)
+        self.assertEqual(p,p_)
+
+    def test_crypto_core_ristretto255_add_sub(self):
+        if not pysodium.sodium_version_check(1, 0, 18): return
+        p = pysodium.crypto_core_ristretto255_random()
+        q = pysodium.crypto_core_ristretto255_random()
+
+        p_q = pysodium.crypto_core_ristretto255_add(p, q)
+        r = pysodium.crypto_core_ristretto255_sub(p_q,q)
+        self.assertEqual(p,r)
+
+    def test_crypto_core_ristretto255_scalar_add_sub(self):
+        if not pysodium.sodium_version_check(1, 0, 18): return
+
+        x = pysodium.crypto_core_ristretto255_scalar_random()
+        y = pysodium.crypto_core_ristretto255_scalar_random()
+        x_y = pysodium.crypto_core_ristretto255_scalar_add(x,y)
+        r = pysodium.crypto_core_ristretto255_scalar_sub(x_y,y)
+
+        p1 = pysodium.crypto_scalarmult_ristretto255_base(x)
+        p2 = pysodium.crypto_scalarmult_ristretto255_base(r)
+        self.assertEqual(p1,p2)
+
+    def test_crypto_core_ristretto255_scalar_negate(self):
+        if not pysodium.sodium_version_check(1, 0, 18): return
+        s = pysodium.crypto_core_ristretto255_scalar_random()
+        r = pysodium.crypto_core_ristretto255_scalar_negate(s)
+        # s + neg(s) = 0 mod L
+        s_r = pysodium.crypto_core_ristretto255_scalar_add(s,r)
+        self.assertEqual(s_r,b"\x00"*32)
+
+    def test_crypto_core_ristretto255_scalar_complement(self):
+        if not pysodium.sodium_version_check(1, 0, 18): return
+        x = pysodium.crypto_core_ristretto255_scalar_random()
+        x_ = pysodium.crypto_core_ristretto255_scalar_complement(x)
+        # x + complement(x) = 1 mod L
+        one = pysodium.crypto_core_ristretto255_scalar_add(x,x_)
+        self.assertEqual(one,b'\x01'+b"\x00"*31)
+
+    def test_crypto_core_ristretto255_scalar_mul(self):
+        if not pysodium.sodium_version_check(1, 0, 18): return
+        two = b'\x02' + b'\x00' * 31
+        four_mul = pysodium.crypto_core_ristretto255_scalar_mul(two,two)
+        four_add = pysodium.crypto_core_ristretto255_scalar_add(two,two)
+        self.assertEqual(four_mul,four_add)
+
+        x = pysodium.crypto_core_ristretto255_scalar_random()
+        one = b'\x01' + b'\x00' * 31
+        r = pysodium.crypto_core_ristretto255_scalar_mul(x,one)
+        self.assertEqual(x,r)
+
 if __name__ == '__main__':
     unittest.main()
